package com.cinema.cinemarental.activities

import android.content.Intent
import android.os.Bundle
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.*
import android.widget.LinearLayout
import androidx.appcompat.app.AppCompatActivity
import com.cinema.cinemarental.R
import com.cinema.cinemarental.model.MessagesModel
import com.cinema.cinemarental.util.References
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.ktx.auth
import com.google.firebase.database.*
import com.google.firebase.ktx.Firebase
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.HashMap


class ChatActivity : AppCompatActivity() {

    private lateinit var reference1: DatabaseReference
    private lateinit var auth: FirebaseAuth

    private lateinit var back: ImageView
    private lateinit var username: TextView
    private lateinit var layout: LinearLayout
    private lateinit var sendButton: ImageView
    private lateinit var messageArea: EditText
    private lateinit var scrollView: ScrollView
    private lateinit var sdf: SimpleDateFormat

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_chat)

        val userData = intent.getStringExtra("user")

        auth = Firebase.auth
        reference1 = References.getDatabaseReference("messages").child("${auth.currentUser?.displayName.toString()}_$userData")

        sdf = SimpleDateFormat("EEE, d MMM '-' HH:mm")

        back = findViewById(R.id.chat_back_button)
        username = findViewById(R.id.chat_username)
        layout = findViewById(R.id.chat_layout1)
        sendButton = findViewById(R.id.chat_sendButton)
        messageArea = findViewById(R.id.chat_messageArea)
        scrollView = findViewById(R.id.chat_scrollView)

        username.text = userData

        back.setOnClickListener(View.OnClickListener {
            finish()
        })

        username.setOnClickListener(View.OnClickListener {
            val intent = Intent(this, ProfileActivity::class.java)
            intent.putExtra("user", userData)
            startActivity(intent)
        })

        sendButton.setOnClickListener(View.OnClickListener {

            if (userData != null) {
                MessagesModel.sendMessage(userData, messageArea)
            }
        })

        reference1.addChildEventListener(object: ChildEventListener {

            override fun onChildAdded(snapshot: DataSnapshot, previousChildName: String?) {
                printMessages(snapshot)
            }

            override fun onChildRemoved(snapshot: DataSnapshot) {

            }

            override fun onCancelled(error: DatabaseError) {

            }

            override fun onChildMoved(snapshot: DataSnapshot, previousChildName: String?) {

            }

            override fun onChildChanged(snapshot: DataSnapshot, previousChildName: String?) {
                printMessages(snapshot)
            }
        })
    }

    private fun printMessages(snapshot: DataSnapshot) {
        val map = snapshot.value as Map<*, *>
        val message = map.get("message").toString()
        val userName = map.get("user").toString()
        val time = map.get("time").toString()

        if (userName.equals(auth.currentUser?.displayName.toString())) {
            addMessageBox(message, time, 1)
        } else {
            addMessageBox(message, time, 2)
        }
    }

    fun addMessageBox(message: String?, time: String?, type: Int) {

        val msgText = TextView(this)
        val timeText = TextView(this)

        msgText.text = message
        timeText.text = time
        timeText.setTextSize(TypedValue.COMPLEX_UNIT_SP, 11f)

        val lp2 = LinearLayout.LayoutParams(
            ViewGroup.LayoutParams.WRAP_CONTENT,
            ViewGroup.LayoutParams.WRAP_CONTENT
        )
        val lp3 = LinearLayout.LayoutParams(
            ViewGroup.LayoutParams.WRAP_CONTENT,
            ViewGroup.LayoutParams.WRAP_CONTENT
        )
        lp2.weight = 1.0f

        if (type == 1) {
            lp2.gravity = Gravity.RIGHT
            lp3.gravity = Gravity.RIGHT
            lp3.setMargins(0,0,0,10)

            msgText.setBackgroundResource(R.drawable.chat_my_message)
        } else {
            lp2.gravity = Gravity.LEFT
            lp3.gravity = Gravity.LEFT
            lp3.setMargins(0,0,0,10)

            msgText.setBackgroundResource(R.drawable.chat_user_message)
        }

        msgText.layoutParams = lp2
        timeText.layoutParams = lp3

        layout.addView(msgText)
        layout.addView(timeText)

        scrollView.fullScroll(View.FOCUS_DOWN)
    }
}