package com.cinema.cinemarental.activities

import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.ImageView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.cinema.cinemarental.R
import com.cinema.cinemarental.adapters.CommentsAdapter
import com.cinema.cinemarental.util.TopSpacingItemDecoration
import com.cinema.cinemarental.data.DtoComments
import com.cinema.cinemarental.util.References
import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import java.util.*
import kotlin.collections.HashMap

class CommentsActivity : AppCompatActivity() {

    lateinit var recyclerView: RecyclerView
    lateinit var commentsAdapter: CommentsAdapter
    
    private var list = ArrayList<DtoComments>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_comments)

        recyclerView = findViewById(R.id.comments_recyclerView)
        commentsAdapter = CommentsAdapter()

        val noCommentsImage: ImageView = findViewById(R.id.comments_no_comments)
        val back: ImageView = findViewById(R.id.comments_back_button)

        val userData = intent.getStringExtra("user")
        val commentsRef = References.getDatabaseReference("comments/$userData")

        commentsRef.addValueEventListener(object: ValueEventListener {

            override fun onDataChange(snapshot: DataSnapshot) {
                list.clear()

                for(item in snapshot.children) {
                    val itemMap: HashMap<String, Objects> = item.value as HashMap<String, Objects>

                    val name: String = itemMap.get("from").toString()
                    val comment: String = itemMap.get("comment").toString()
                    val date: String = itemMap.get("date").toString()
                    val rating: Double = java.lang.Double.parseDouble(itemMap.get("rating").toString())

                    list.add(DtoComments(name, comment, date, rating))
                }

                if(list.isNotEmpty()) noCommentsImage.visibility = View.GONE

                initRecyclerView()
                commentsAdapter.submitList(list)
            }

            override fun onCancelled(error: DatabaseError) {
                Log.d(TAG, error.message)
            }
        })

        back.setOnClickListener(View.OnClickListener {
            finish()
        })
    }

    private fun initRecyclerView() {
        recyclerView.apply {
            layoutManager = LinearLayoutManager(context)
            val topSpacingDecoration = TopSpacingItemDecoration(30)
            addItemDecoration(topSpacingDecoration)
            commentsAdapter = CommentsAdapter()
            adapter = commentsAdapter
        }
    }

    companion object {
        const val TAG = "COMMENTS"
    }
}