package com.cinema.cinemarental.activities

import android.Manifest
import android.app.Activity
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.drawable.BitmapDrawable
import android.os.Build
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.View
import android.widget.*
import androidx.core.app.ActivityCompat
import androidx.core.graphics.drawable.toBitmap
import com.cinema.cinemarental.R
import com.cinema.cinemarental.data.DtoRentals
import com.cinema.cinemarental.model.RentalModel
import com.cinema.cinemarental.model.StorageModel
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import com.google.android.gms.maps.model.LatLng
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.ktx.auth
import com.google.firebase.ktx.Firebase

class NewRentalActivity : AppCompatActivity() {

    lateinit var auth: FirebaseAuth

    lateinit var back: ImageView
    lateinit var upload: ImageView
    lateinit var title: EditText
    lateinit var description: EditText
    lateinit var category: Spinner
    lateinit var price: EditText

    lateinit var myImage1: ImageView
    lateinit var myImage2: ImageView
    lateinit var myImage3: ImageView
    lateinit var myImage4: ImageView

    private lateinit var fusedLocationClient: FusedLocationProviderClient
    private lateinit var myLocation: LatLng

    var imageSelected: Boolean = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_new_rental)

        fusedLocationClient = LocationServices.getFusedLocationProviderClient(this)
        getCurrentLocation()

        auth = Firebase.auth

        back = findViewById(R.id.new_rental_back_button)
        upload = findViewById(R.id.new_rental_upload)
        title = findViewById(R.id.new_rental_title)
        description = findViewById(R.id.new_rental_description)
        category = findViewById(R.id.new_rental_category_spinner)
        price = findViewById(R.id.new_rental_price)


        myImage1 = findViewById(R.id.image1)
        myImage2 = findViewById(R.id.image2)
        myImage3 = findViewById(R.id.image3)
        myImage4 = findViewById(R.id.image4)

        val catSpinner: Spinner = findViewById(R.id.new_rental_category_spinner)
        ArrayAdapter.createFromResource(this, R.array.category_array, android.R.layout.simple_spinner_item).also { adapter ->
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
            catSpinner.adapter = adapter
        }

        back.setOnClickListener(View.OnClickListener {
            finish()
        })

        upload.setOnClickListener(View.OnClickListener {
            if(imageSelected) uploadProduct()
            else Toast.makeText(applicationContext, "At least one image is required", Toast.LENGTH_LONG).show()
        })

        myImage1.setOnClickListener(View.OnClickListener {
            clickOnImage(1)
        })

        myImage2.setOnClickListener(View.OnClickListener {
            clickOnImage(2)
        })
        myImage3.setOnClickListener(View.OnClickListener {
            clickOnImage(3)
        })

        myImage4.setOnClickListener(View.OnClickListener {
            clickOnImage(4)
        })
    }

    private fun uploadProduct() {

        if(!myImage1.drawable.toBitmap().isMutable) {
            StorageModel.saveImage((myImage1.drawable as BitmapDrawable).bitmap, 1, title.text.toString())
        }
        if(!myImage2.drawable.toBitmap().isMutable) {
            StorageModel.saveImage((myImage2.drawable as BitmapDrawable).bitmap, 2, title.text.toString())
        }
        if(!myImage3.drawable.toBitmap().isMutable) {
            StorageModel.saveImage((myImage3.drawable as BitmapDrawable).bitmap, 3, title.text.toString())
        }
        if(!myImage4.drawable.toBitmap().isMutable) {
            StorageModel.saveImage((myImage4.drawable as BitmapDrawable).bitmap, 4, title.text.toString())
        }

        val dtoRentals = DtoRentals(auth.currentUser?.displayName.toString(), title.text.toString(), description.text.toString(), price.text.toString(), myImage1.drawable.toString(), myLocation.latitude, myLocation.longitude)

        RentalModel.saveRental(dtoRentals)

        Toast.makeText(applicationContext, "Product uploaded successfully ", Toast.LENGTH_LONG).show()
        finish()

    }

    private fun clickOnImage(image: Int) {
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if(checkSelfPermission(Manifest.permission.READ_EXTERNAL_STORAGE) == PackageManager.PERMISSION_DENIED) {
                var permissions = arrayOf(Manifest.permission.READ_EXTERNAL_STORAGE)
                requestPermissions(permissions, PERMISSION_CODE)
            } else {
                pickImageFromGallery(image)
            }
        } else {
            pickImageFromGallery(image)
        }
    }

    private fun pickImageFromGallery(image: Int) {
        val intent = Intent(Intent.ACTION_PICK)
        intent.type = "image/*"

        when(image) {
            1 -> startActivityForResult(intent, IMAGE1_PICK_CODE)
            2 -> startActivityForResult(intent, IMAGE2_PICK_CODE)
            3 -> startActivityForResult(intent, IMAGE3_PICK_CODE)
            4 -> startActivityForResult(intent, IMAGE4_PICK_CODE)
        }

    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        when(requestCode) {
            PERMISSION_CODE -> {
                if(grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    pickImageFromGallery(1)
                } else {
                    Toast.makeText(applicationContext, "Permission denied", Toast.LENGTH_SHORT).show()
                }
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if(resultCode == Activity.RESULT_OK)
            when(requestCode) {
                IMAGE1_PICK_CODE -> myImage1.setImageURI(data?.data)
                IMAGE2_PICK_CODE -> myImage2.setImageURI(data?.data)
                IMAGE3_PICK_CODE -> myImage3.setImageURI(data?.data)
                IMAGE4_PICK_CODE -> myImage4.setImageURI(data?.data)
            }

        imageSelected = true
    }

    private fun getCurrentLocation()  {
        checkPermission()
        fusedLocationClient.lastLocation.addOnSuccessListener {location ->
            myLocation = LatLng(location.latitude, location.longitude)
        }
    }

    private fun checkPermission() {
        if(ActivityCompat.checkSelfPermission(this,
                Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this,
                arrayOf(Manifest.permission.ACCESS_FINE_LOCATION),
                LOCATION_PERMISSION_REQUEST_CODE
            )
            return
        }
    }
    companion object {
        const val IMAGE1_PICK_CODE = 1
        const val IMAGE2_PICK_CODE = 2
        const val IMAGE3_PICK_CODE = 3
        const val IMAGE4_PICK_CODE = 4
        const val PERMISSION_CODE = 1001
        const val LOCATION_PERMISSION_REQUEST_CODE = 10
    }
}