package com.cinema.cinemarental.activities

import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.ProgressBar
import android.widget.Toast
import com.cinema.cinemarental.R
import com.cinema.cinemarental.ui.login.afterTextChanged
import com.google.firebase.FirebaseApp
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.UserProfileChangeRequest
import com.google.firebase.auth.ktx.auth
import com.google.firebase.ktx.Firebase

class NewUserActivity : AppCompatActivity() {

    private var progressBar: ProgressBar? = null
    private lateinit var auth: FirebaseAuth

    lateinit var name: EditText
    lateinit var surname: EditText
    lateinit var email: EditText
    lateinit var password: EditText
    lateinit var register: Button
    lateinit var cancel: Button

    override fun onCreate(savedInstanceState: Bundle?) {

        FirebaseApp.initializeApp(this)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_new_user)

        auth = Firebase.auth

        name = findViewById(R.id.name)
        surname = findViewById(R.id.surname)
        email = findViewById(R.id.email)
        password = findViewById(R.id.password)
        register = findViewById(R.id.register)
        cancel = findViewById(R.id.cancel)

        name.afterTextChanged { validateFields() }
        surname.afterTextChanged { validateFields() }
        email.afterTextChanged { validateFields() }
        password.afterTextChanged { validateFields() }

        register.setOnClickListener(View.OnClickListener {
            createAccount(email.text.toString(), password.text.toString(), name.text.toString(), surname.text.toString())
        })
        cancel.setOnClickListener(View.OnClickListener {
            finish()
        })
    }

    private fun createAccount(email: String, password: String, name: String, surname: String) {
        Log.d(TAG, "createAccount:$email")
        showProgressBar()

        auth.createUserWithEmailAndPassword(email, password)
            .addOnCompleteListener(this) { task ->
                if (task.isSuccessful) {
                    Log.d(TAG, "createUserWithEmail:success")
                    val user = auth.currentUser
                    val profile: UserProfileChangeRequest = UserProfileChangeRequest.Builder().setDisplayName("$name $surname").build()
                    user?.updateProfile(profile)
                    val intent = Intent(this, MainActivity::class.java)
                    startActivity(intent)
                } else {
                    Log.w(TAG, "createUserWithEmail:failure", task.exception)
                    Toast.makeText(baseContext, "Authentication failed.",
                        Toast.LENGTH_SHORT).show()
                }

                hideProgressBar()
            }
    }

    private fun validateFields() {
        register.isEnabled = name.text.toString().isNotEmpty() && surname.text.toString().isNotEmpty() && email.text.toString().isNotEmpty() && password.text.toString().isNotEmpty()
    }

    private fun showProgressBar() {
        progressBar?.visibility = View.VISIBLE
    }

    private fun hideProgressBar() {
        progressBar?.visibility = View.INVISIBLE
    }

    companion object {
        private const val TAG = "EmailPassword"
    }
}