package com.cinema.cinemarental.activities

import android.app.Activity
import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.View
import android.view.inputmethod.InputMethodManager
import android.widget.*
import com.cinema.cinemarental.R
import com.cinema.cinemarental.glide.GlideApp
import com.cinema.cinemarental.model.UserModel
import com.cinema.cinemarental.ui.login.afterTextChanged
import com.cinema.cinemarental.util.References
import com.google.firebase.storage.StorageReference
import kotlinx.coroutines.runBlocking

class ProfileActivity : AppCompatActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_profile)

        var storageRef: StorageReference = References.getStorageReference()

        val user = intent.getStringExtra("user")

        val back: ImageView = findViewById(R.id.activity_profile_back_button)
        val image = findViewById<ImageView>(R.id.activity_profile_image)
        val name = findViewById<TextView>(R.id.activity_profile_name)
        val ratingAverage = findViewById<RatingBar>(R.id.activity_profile_ratingBar)
        var layout: LinearLayout = findViewById(R.id.activity_profile_layout_comments)
        val numComments = findViewById<TextView>(R.id.activity_profile_num_comments)
        val textArea: EditText = findViewById(R.id.activity_profile_textarea)
        val ratingBar: RatingBar = findViewById(R.id.activity_profile_ratingBar_2)
        val comment: Button = findViewById(R.id.activity_profile_comment)

        name.text = user

        UserModel.getRatingAverage(user.toString(), ratingAverage, numComments)

        runBlocking {
            var imagesRef: StorageReference = storageRef.child("${user}/profile")
            GlideApp.with(applicationContext).load(imagesRef).into(image)
        }

        textArea.afterTextChanged {
            comment.isEnabled = !textArea.text.isEmpty()
        }

        textArea.onFocusChangeListener = View.OnFocusChangeListener { view, hasFocus ->
            if (!hasFocus ) {
                hideKeyboard(view)
            }
        }

        layout.setOnClickListener(View.OnClickListener {
            val intent = Intent(this, CommentsActivity::class.java)
            intent.putExtra("user", user)
            startActivity(intent)
        })


        comment.setOnClickListener(View.OnClickListener {
            val numStars = ratingBar.rating

            UserModel.sendComment(user.toString(), numStars, textArea.text.toString())

            Toast.makeText(this, "Comment sended", Toast.LENGTH_SHORT).show()
        })

        back.setOnClickListener(View.OnClickListener {
            finish()
        })
    }

    private fun hideKeyboard(view: View) {
        val inputMethodManager: InputMethodManager = getSystemService(Activity.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.hideSoftInputFromWindow(view.windowToken, 0)
    }
}