package com.cinema.cinemarental.activities

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.view.inputmethod.InputMethodManager
import android.widget.*
import androidx.appcompat.app.AppCompatActivity
import com.cinema.cinemarental.R
import com.cinema.cinemarental.glide.GlideApp
import com.cinema.cinemarental.model.UserModel
import com.cinema.cinemarental.ui.login.afterTextChanged
import com.cinema.cinemarental.util.References
import com.google.android.gms.tasks.OnSuccessListener
import com.google.firebase.storage.ListResult
import com.google.firebase.storage.StorageReference


class RentalDetailActivity : AppCompatActivity() {

    lateinit var image1: ImageView
    lateinit var image2: ImageView
    lateinit var image3: ImageView
    lateinit var image4: ImageView
    lateinit var title: EditText
    lateinit var description: EditText
    lateinit var ratingAverage : RatingBar

    lateinit var storageRef: StorageReference

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_rental_detail)

        storageRef = References.getStorageReference()

        val back: ImageView = findViewById(R.id.rental_details_back_button)
        image1 = findViewById(R.id.rental_details_image1)
        image2 = findViewById(R.id.rental_details_image2)
        image3 = findViewById(R.id.rental_details_image3)
        image4 = findViewById(R.id.rental_details_image4)
        val title: TextView = findViewById(R.id.rental_details_title)
        val description: TextView = findViewById(R.id.rental_details_description)
        val price: TextView = findViewById(R.id.rental_details_price)
        val name: TextView = findViewById(R.id.rental_details_name)
        ratingAverage = findViewById(R.id.rental_details_ratingBar)
        val layout: LinearLayout = findViewById(R.id.rental_details_layout_comments)
        val numComments: TextView = findViewById(R.id.rental_details_num_comments)
        val chat: Button = findViewById(R.id.rental_details_chat)
        val textArea: EditText = findViewById(R.id.rental_details_textarea)
        val ratingBar: RatingBar = findViewById(R.id.rental_details_ratingBar_2)
        val comment: Button = findViewById(R.id.rental_details_comment)

        val dataTitle = intent.getStringExtra("title")
        val dataDescription = intent.getStringExtra("description")
        val dataPrice = intent.getStringExtra("price")
        val dataUser = intent.getStringExtra("user")

        title.text = dataTitle
        description.text = dataDescription
        price.text = dataPrice
        name.text = dataUser

        UserModel.getRatingAverage(dataUser.toString(), ratingAverage, numComments)
        checkMoreImages(dataUser.toString(), dataTitle.toString())

        textArea.afterTextChanged {
            comment.isEnabled = !textArea.text.isEmpty()
        }

        textArea.onFocusChangeListener = View.OnFocusChangeListener { view, hasFocus ->
            if (!hasFocus ) {
                hideKeyboard(view)
            }
        }

        layout.setOnClickListener(View.OnClickListener {
            val intent = Intent(this, CommentsActivity::class.java)
            intent.putExtra("user", dataUser)
            startActivity(intent)
        })

        comment.setOnClickListener(View.OnClickListener {
            val numStars = ratingBar.rating

            UserModel.sendComment(dataUser.toString(), numStars, textArea.text.toString())

            Toast.makeText(this, "Comment sended", Toast.LENGTH_SHORT).show()
        })

        chat.setOnClickListener(View.OnClickListener {
            val intent = Intent(this, ChatActivity::class.java)
            intent.putExtra("user", dataUser)
            intent.putExtra("item", dataTitle)
            startActivity(intent)
        })

        back.setOnClickListener(View.OnClickListener {
            finish()
        })
    }

    private fun hideKeyboard(view: View) {
        val inputMethodManager: InputMethodManager = getSystemService(Activity.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.hideSoftInputFromWindow(view.windowToken, 0)
    }


    private fun checkMoreImages(dataUser: String, title: String) {
        var cont = 1
        var imagesRef = storageRef.child("$dataUser/${title}")

        imagesRef.listAll()
            .addOnSuccessListener(OnSuccessListener<ListResult> { listResult ->
                for (item in listResult.items) {
                    var pathRef: StorageReference
                    when(cont) {
                        1 -> {
                            image1.visibility = View.VISIBLE
                            pathRef = storageRef.child(item.path)
                            setImages(image1, pathRef)
                        }
                        2 -> {
                            image2.visibility = View.VISIBLE
                            pathRef = storageRef.child(item.path)
                            setImages(image2, pathRef)
                        }
                        3 -> {
                            image3.visibility = View.VISIBLE
                            pathRef = storageRef.child(item.path)
                            setImages(image3, pathRef)
                        }
                        4 -> {
                            image4.visibility = View.VISIBLE
                            pathRef = storageRef.child(item.path)
                            setImages(image4, pathRef)
                        }
                    }
                    cont++
                }
            }).addOnFailureListener { }
    }

    private fun setImages(imageView: ImageView, imageRef: StorageReference) {
        GlideApp.with(applicationContext).load(imageRef).into(imageView)
    }

    companion object {
        const val TAG = "RENTALS_DETAIL"
    }
}