package com.cinema.cinemarental.activities.ui.main

import android.Manifest
import android.app.Activity
import android.app.AlertDialog
import android.content.DialogInterface
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.drawable.BitmapDrawable
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.core.app.ActivityCompat
import androidx.fragment.app.Fragment
import com.cinema.cinemarental.R
import com.cinema.cinemarental.activities.CommentsActivity
import com.cinema.cinemarental.glide.GlideApp
import com.cinema.cinemarental.model.StorageModel
import com.cinema.cinemarental.model.UserModel
import com.cinema.cinemarental.ui.login.LoginActivity
import com.cinema.cinemarental.util.References
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.ktx.auth
import com.google.firebase.ktx.Firebase
import com.google.firebase.storage.StorageReference
import kotlinx.coroutines.runBlocking


/**
 * A simple [Fragment] subclass
 */
class ProfileFragment : Fragment() {

    private lateinit var auth: FirebaseAuth

    private lateinit var ratingAverage : RatingBar
    private lateinit var numComments : TextView
    private lateinit var image: ImageView

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        auth = Firebase.auth

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {

        val view: View = inflater.inflate(R.layout.fragment_profile, container, false)

        var storageRef: StorageReference = References.getStorageReference()

        image = view.findViewById(R.id.profile_imageView)
        ratingAverage = view.findViewById(R.id.profile_ratingBar)
        numComments = view.findViewById(R.id.profile_num_comments)
        val comments = view.findViewById<TextView>(R.id.profile_comments)
        val name = view.findViewById<TextView>(R.id.name_profile)
        val email = view.findViewById<TextView>(R.id.email)
        val signOff = view.findViewById<Button>(R.id.profile_signoff)

        runBlocking {
            var imagesRef: StorageReference = storageRef.child("${auth.currentUser.displayName}/profile")
            context?.let { GlideApp.with(it).load(imagesRef).into(image) }
        }

        name.text = auth.currentUser?.displayName.toString()
        email.text = auth.currentUser?.email.toString()

        UserModel.getRatingAverage(auth.currentUser?.displayName.toString(), ratingAverage, numComments)

        image.setOnClickListener(View.OnClickListener {
            clickOnImage()
        })

        comments.setOnClickListener(View.OnClickListener {
            val intent = Intent(context, CommentsActivity::class.java)
            intent.putExtra("user", auth.currentUser?.displayName.toString())
            startActivity(intent)
        })

        signOff.setOnClickListener(View.OnClickListener {
            val builder = showDialog(view).create()
            builder.show()
        })

        return view
    }

    private fun showDialog(view: View): AlertDialog.Builder {
        val builder = AlertDialog.Builder(view.context)

        builder.setTitle("Confirm")
        builder.setMessage("Are you sure to sign off?")

        builder.setPositiveButton("YES", object : DialogInterface.OnClickListener {
            override fun onClick(dialog: DialogInterface, which: Int) {
                auth.signOut()
                val intent = Intent(context, LoginActivity::class.java)
                startActivity(intent)
                dialog.dismiss()
            }
        })

        builder.setNegativeButton("NO", object : DialogInterface.OnClickListener {
            override fun onClick(dialog: DialogInterface, which: Int) {

                dialog.dismiss()
            }
        })

        return builder
    }

    private fun clickOnImage() {
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if(ActivityCompat.checkSelfPermission(requireContext(), Manifest.permission.READ_EXTERNAL_STORAGE) == PackageManager.PERMISSION_DENIED) {
                var permissions = arrayOf(Manifest.permission.READ_EXTERNAL_STORAGE)
                requestPermissions(permissions, PERMISSION_CODE)
            } else {
                pickImageFromGallery()
            }
        } else {
            pickImageFromGallery()
        }
    }

    private fun pickImageFromGallery() {
        val intent = Intent(Intent.ACTION_PICK)
        intent.type = "image/*"

        startActivityForResult(intent, 1)
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        when(requestCode) {
            PERMISSION_CODE -> {
                if(grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    pickImageFromGallery()
                } else {
                    Toast.makeText(context, "Permission denied", Toast.LENGTH_SHORT).show()
                }
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if(resultCode == Activity.RESULT_OK) {
            image.setImageURI(data?.data)
            StorageModel.saveImage((image.drawable as BitmapDrawable).bitmap, 0, "")
        }
    }

    companion object {
        const val TAG = "PROFILE"
        const val PERMISSION_CODE = 1001
    }
}