package com.cinema.cinemarental.activities.ui.main

import android.Manifest
import android.content.pm.PackageManager
import android.os.Bundle
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.RadioButton
import android.widget.RadioGroup
import androidx.core.app.ActivityCompat
import androidx.recyclerview.widget.RecyclerView
import com.cinema.cinemarental.R
import com.cinema.cinemarental.model.RentalModel
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import com.google.android.gms.maps.model.LatLng

/**
 * A simple [Fragment] subclass
 */
class RentalsFragment : Fragment() {

    lateinit var recyclerView: RecyclerView
    lateinit var noRentalsImage: ImageView

    private lateinit var fusedLocationClient: FusedLocationProviderClient
    private lateinit var myLocation: LatLng

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        fusedLocationClient = LocationServices.getFusedLocationProviderClient(context)
        getCurrentLocation()
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {

        val view: View = inflater.inflate(R.layout.fragment_rentals, container, false)
        val radioGroup: RadioGroup = view.findViewById(R.id.rentals_radiogroup)

        noRentalsImage = view.findViewById(R.id.rentals_no_rentals)
        recyclerView = view.findViewById(R.id.rentals_recyclerView)

        radioGroup.setOnCheckedChangeListener { group, checkedId ->
            val radio: RadioButton = group.findViewById(checkedId)
            when(radio.text) {
                activity?.resources?.getString(R.string.radio_all) -> {
                    RentalModel.getAllRentals(recyclerView, noRentalsImage)
                }
                activity?.resources?.getString(R.string.radio_20) -> {
                    RentalModel.getRentalsWithFilter(recyclerView, noRentalsImage, 20, myLocation)
                }
                activity?.resources?.getString(R.string.radio_50) -> {
                    RentalModel.getRentalsWithFilter(recyclerView, noRentalsImage, 50, myLocation)
                }
            }
        }

        RentalModel.getAllRentals(recyclerView, noRentalsImage)

        return view
    }

    private fun getCurrentLocation()  {
        checkPermission()
        fusedLocationClient.lastLocation.addOnSuccessListener {location ->
            if (location != null) {
                myLocation = LatLng(location.latitude, location.longitude)
            }
        }
    }

    private fun checkPermission() {
        if(ActivityCompat.checkSelfPermission(requireContext(),
                Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(requireActivity(),
                arrayOf(Manifest.permission.ACCESS_FINE_LOCATION),
                LOCATION_PERMISSION_REQUEST_CODE
            )
            return
        }
    }

    companion object {
        private const val LOCATION_PERMISSION_REQUEST_CODE = 10
        const val TAG = "RENTALS"
    }
}