package com.cinema.cinemarental.adapters

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RatingBar
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.core.content.ContextCompat.startActivity
import androidx.recyclerview.widget.RecyclerView
import com.cinema.cinemarental.R
import com.cinema.cinemarental.activities.ProfileActivity
import com.cinema.cinemarental.data.DtoComments
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.ktx.auth
import com.google.firebase.ktx.Firebase


class CommentsAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    private var items: List<DtoComments> = ArrayList()

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return CommentsViewHolder(
                LayoutInflater.from(parent.context).inflate(R.layout.item_list_comments, parent, false)
        )
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        when(holder) {
            is CommentsViewHolder -> {
                holder.bind(items.get(position))
            }
        }
    }

    override fun getItemCount(): Int {
        return items.size
    }

    fun submitList(rentalsList: List<DtoComments>) {
        items = rentalsList
    }

    class CommentsViewHolder constructor(
            itemView: View
    ): RecyclerView.ViewHolder(itemView) {

        var auth: FirebaseAuth = Firebase.auth

        val from: TextView = itemView.findViewById(R.id.item_list_comments_by_value)
        val date: TextView = itemView.findViewById(R.id.item_list_comments_date_value)
        val comment: TextView = itemView.findViewById(R.id.item_list_comments_description)
        val ratingBar: RatingBar = itemView.findViewById(R.id.item_list_comments_ratingbar)

        fun bind(dtoComments: DtoComments) {
            from.text = dtoComments.from
            date.text = dtoComments.date
            comment.text = dtoComments.comment
            ratingBar.setRating(dtoComments.rating.toFloat())

            from.setOnClickListener(View.OnClickListener {
                if(!auth.currentUser.displayName.equals(dtoComments.from)) {
                    val intent = Intent(itemView.context, ProfileActivity::class.java)
                    intent.putExtra("user", dtoComments.from)
                    val extras: Bundle? = intent.extras
                    startActivity(itemView.context, intent, extras)
                }

            })
        }

    }
}