package com.cinema.cinemarental.adapters

import android.content.Intent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.cinema.cinemarental.R
import com.cinema.cinemarental.activities.ChatActivity
import com.cinema.cinemarental.data.DtoMessages
import com.cinema.cinemarental.model.MessagesModel
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.ktx.auth
import com.google.firebase.ktx.Firebase

class MessagesAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    private var items: List<DtoMessages> = ArrayList()


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return MessagesViewHolder(
                LayoutInflater.from(parent.context).inflate(R.layout.item_list_messages, parent, false)
        )
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        when(holder) {
            is MessagesViewHolder -> {
                holder.bind(items.get(position))
            }
        }
    }

    override fun getItemCount(): Int {
        return items.size
    }

    fun submitList(rentalsList: List<DtoMessages>) {
        items = rentalsList
    }

    class MessagesViewHolder constructor(
            itemView: View
    ): RecyclerView.ViewHolder(itemView) {

        var auth: FirebaseAuth = Firebase.auth

        val layout: LinearLayout = itemView.findViewById(R.id.item_list_messages_layout)
        val image: ImageView = itemView.findViewById(R.id.item_list_messages_image)
        val user: TextView = itemView.findViewById(R.id.item_list_messages_user)
        val message: TextView = itemView.findViewById(R.id.item_list_messages_comment)
        val time: TextView = itemView.findViewById(R.id.item_list_messages_time)

        fun bind(dtoMessages: DtoMessages) {
            user.text = dtoMessages.chatWith
            message.text = dtoMessages.lastMessage
            time.text = dtoMessages.lastTime

            if(dtoMessages.unread.equals("true")) image.setImageResource(R.mipmap.ic_messages_notification_foreground)
            else image.setImageResource(R.mipmap.ic_no_messages_notification_foreground)

            layout.setOnClickListener(View.OnClickListener {
                updateUnreadedMessages(dtoMessages.chatWith)
                image.setImageResource(R.mipmap.ic_no_messages_notification_foreground)
                val intent = Intent(itemView.context, ChatActivity::class.java)
                intent.putExtra("user", dtoMessages.chatWith)
                itemView.context.startActivity(intent)
            })
        }

        private fun updateUnreadedMessages(chatWith: String) {
            MessagesModel.updateUnreadedMessages(chatWith)
        }
    }
}