package com.cinema.cinemarental.adapters

import android.app.AlertDialog
import android.content.DialogInterface
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.cinema.cinemarental.R
import com.cinema.cinemarental.data.DtoRentals
import com.cinema.cinemarental.util.References
import com.cinema.cinemarental.glide.GlideApp
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.ktx.auth
import com.google.firebase.ktx.Firebase
import com.google.firebase.storage.StorageReference
import kotlinx.coroutines.runBlocking

class MyRentalsAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    private var items: List<DtoRentals> = ArrayList()


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return MyRentalsViewHolder(
                LayoutInflater.from(parent.context).inflate(R.layout.item_list_my_rentals, parent, false)
        )
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        when(holder) {
            is MyRentalsViewHolder -> {
                holder.bind(items.get(position))
            }
        }
    }

    override fun getItemCount(): Int {
        return items.size
    }

    fun submitList(rentalsList: List<DtoRentals>) {
        items = rentalsList
    }

    class MyRentalsViewHolder constructor(
            itemView: View
    ): RecyclerView.ViewHolder(itemView) {

        var auth: FirebaseAuth = Firebase.auth
        var storageRef: StorageReference = References.getStorageReference()

        val title: TextView = itemView.findViewById(R.id.item_list_my_rentals_title)
        val description: TextView = itemView.findViewById(R.id.item_list_my_rentals_description)
        val image: ImageView = itemView.findViewById(R.id.item_list_my_rentals_image)
        val trash: ImageView = itemView.findViewById(R.id.item_list_my_rentals_remove_item)

        fun bind(dtoRentals: DtoRentals) {
            title.text = dtoRentals.title
            description.text = dtoRentals.description

            runBlocking {
                var imagesRef: StorageReference = storageRef.child("${auth.currentUser?.displayName}/${title.text}/image1")
                GlideApp.with(itemView.context).asBitmap().centerCrop().load(imagesRef).into(image)
            }

            trash.setOnClickListener(View.OnClickListener {
                val builder = showDialog(itemView, title.text.toString()).create()
                builder.show()
            })
        }

        private fun removeItem(title: String) {
            var itemRef = References.getDatabaseReference("rentals/${auth.currentUser?.displayName.toString()}/$title")
            itemRef.setValue(null)
        }

        private fun showDialog(view: View, item: String): AlertDialog.Builder {
            val builder = AlertDialog.Builder(view.context)

            builder.setTitle("Confirm")
            builder.setMessage("Are you sure to remove [$item]?")

            builder.setPositiveButton("YES", object : DialogInterface.OnClickListener {
                override fun onClick(dialog: DialogInterface, which: Int) {
                    removeItem(item)
                    dialog.dismiss()
                }
            })

            builder.setNegativeButton("NO", object : DialogInterface.OnClickListener {
                override fun onClick(dialog: DialogInterface, which: Int) {
                    dialog.dismiss()
                }
            })
            return builder
        }
    }
}