package com.cinema.cinemarental.adapters

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.cinema.cinemarental.R
import com.cinema.cinemarental.activities.RentalDetailActivity
import com.cinema.cinemarental.data.DtoRentals
import com.cinema.cinemarental.util.References
import com.cinema.cinemarental.glide.GlideApp
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.ktx.auth
import com.google.firebase.ktx.Firebase
import com.google.firebase.storage.StorageReference
import kotlinx.coroutines.runBlocking

class RentalsAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    private var items: List<DtoRentals> = ArrayList()

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return MyRentalsViewHolder(
                LayoutInflater.from(parent.context).inflate(R.layout.item_list_rentals, parent, false)
        )
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        when(holder) {
            is MyRentalsViewHolder -> {
                holder.bind(items.get(position))
            }
        }
    }

    override fun getItemCount(): Int {
        return items.size
    }

    fun submitList(rentalsList: List<DtoRentals>) {
        items = rentalsList
    }

    class MyRentalsViewHolder constructor(
            itemView: View
    ): RecyclerView.ViewHolder(itemView) {

        var auth: FirebaseAuth = Firebase.auth
        var storageRef: StorageReference = References.getStorageReference()

        val layout: LinearLayout = itemView.findViewById(R.id.item_list_rentals_layout)
        val title: TextView = itemView.findViewById(R.id.item_list_rentals_title)
        val description: TextView = itemView.findViewById(R.id.item_list_rentals_description)
        val image: ImageView = itemView.findViewById(R.id.item_list_rentals_image)
        val price: TextView = itemView.findViewById(R.id.item_list_rentals_price_value)

        fun bind(dtoRentals: DtoRentals) {
            title.text = dtoRentals.title
            description.text = dtoRentals.description
            price.text = dtoRentals.price
            val user = dtoRentals.owner

            runBlocking {
                var imagesRef: StorageReference = storageRef.child("$user/${title.text}/image1")
                GlideApp.with(itemView.context).load(imagesRef).into(image)
            }

            layout.setOnClickListener(View.OnClickListener {
                val intent = Intent(itemView.context, RentalDetailActivity::class.java)
                intent.putExtra("title", title.text.toString())
                intent.putExtra("description", description.text.toString())
                intent.putExtra("price", price.text.toString())
                intent.putExtra("user", user)
                val extras: Bundle? = intent.extras
                ContextCompat.startActivity(itemView.context, intent, extras)
            })
        }
    }
}