package com.cinema.cinemarental.model

import android.annotation.SuppressLint
import android.view.View
import android.widget.EditText
import android.widget.ImageView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.cinema.cinemarental.adapters.MessagesAdapter
import com.cinema.cinemarental.data.DtoMessages
import com.cinema.cinemarental.util.References
import com.cinema.cinemarental.util.TopSpacingItemDecoration
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.ktx.auth
import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.google.firebase.ktx.Firebase
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.HashMap

class MessagesModel {
    companion object {

        private var auth: FirebaseAuth = Firebase.auth
        lateinit var messagesAdapter: MessagesAdapter

        fun getConversations(recyclerView: RecyclerView, noMessagesImage: ImageView) {
            val messagesRef = References.getDatabaseReference("messages")
            var list = ArrayList<DtoMessages>()

            messagesRef.addValueEventListener(object: ValueEventListener {

                override fun onDataChange(snapshot: DataSnapshot) {
                    list.clear()

                    for(item in snapshot.children) {
                        if(item.key?.startsWith(auth.currentUser?.displayName.toString() + "_", true)!!) {
                            var chatWith = item.key.toString().split("_").toTypedArray()[1]
                            var lastMessage = ""
                            var lastTime = ""
                            var unread = ""

                            for(itemMap in item.children) {
                                var map = itemMap.value as HashMap<String, Objects>
                                lastMessage = map.get("message").toString()
                                lastTime = map.get("time").toString()
                                unread = map.get("unread").toString()
                            }
                            list.add(DtoMessages(chatWith, lastMessage, lastTime, unread))
                        }
                    }

                    if(list.isNotEmpty()) noMessagesImage.visibility = View.GONE
                    else noMessagesImage.visibility = View.VISIBLE

                    initRecyclerView(recyclerView)
                    messagesAdapter.submitList(list)
                }

                override fun onCancelled(error: DatabaseError) {
                    TODO("Not yet implemented")
                }
            })
        }

        @SuppressLint("SimpleDateFormat")
        fun sendMessage(userData: String, messageArea: EditText) {
            var reference1 = References.getDatabaseReference("messages").child("${auth.currentUser?.displayName.toString()}_$userData")
            var reference2 = References.getDatabaseReference("messages").child("${userData}_${auth.currentUser?.displayName.toString()}")

            var sdf = SimpleDateFormat("EEE, d MMM '-' HH:mm")

            val messageText: String = messageArea.text.toString()

            if(messageText.isNotEmpty()) {
                var map1: HashMap<String, String> = HashMap()
                val currentDateAndTime = sdf.format(Date()).toString()

                map1.put("message", messageText)
                map1.put("user", auth.currentUser?.displayName.toString())
                map1.put("time", currentDateAndTime)
                map1.put("unread", "false")


                var map2: HashMap<String, String> = HashMap()

                map2.put("message", messageText)
                map2.put("user", auth.currentUser?.displayName.toString())
                map2.put("time", currentDateAndTime)
                map2.put("unread", "true")

                reference1.push().setValue(map1)
                reference2.push().setValue(map2)
                messageArea.setText("")
            }
        }

        fun updateUnreadedMessages(chatWith: String) {
            var chatRef =References.getDatabaseReference("messages").child("${auth.currentUser?.displayName.toString()}_$chatWith")
            chatRef.orderByKey().limitToLast(1).addListenerForSingleValueEvent(object: ValueEventListener{
                override fun onCancelled(error: DatabaseError) {
                    TODO("Not yet implemented")
                }

                override fun onDataChange(snapshot: DataSnapshot) {
                    for(item in snapshot.children) {
                        var map: HashMap<String, HashMap<String, String>> = HashMap()

                        var mapValue = item.value as HashMap<String, String>
                        mapValue.put("unread", "false")

                        map.put(item.key.toString(), mapValue)

                        chatRef.updateChildren(map as Map<String, Any>)
                    }
                }
            })
        }

        private fun initRecyclerView(recyclerView: RecyclerView) {
            recyclerView.apply {
                layoutManager = LinearLayoutManager(context)
                val topSpacingDecoration = TopSpacingItemDecoration(10)
                addItemDecoration(topSpacingDecoration)
                messagesAdapter = MessagesAdapter()
                adapter = messagesAdapter
            }
        }
    }
}