package com.cinema.cinemarental.model

import android.location.Location
import android.util.Log
import android.view.View
import android.widget.ImageView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.cinema.cinemarental.activities.ui.main.MyRentalsFragment
import com.cinema.cinemarental.activities.ui.main.RentalsFragment
import com.cinema.cinemarental.adapters.MyRentalsAdapter
import com.cinema.cinemarental.adapters.RentalsAdapter
import com.cinema.cinemarental.data.DtoRentals
import com.cinema.cinemarental.util.References
import com.cinema.cinemarental.util.TopSpacingItemDecoration
import com.google.android.gms.maps.model.LatLng
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.ktx.auth
import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.DatabaseReference
import com.google.firebase.database.ValueEventListener
import com.google.firebase.ktx.Firebase
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

class RentalModel {
    companion object {

        private var database: DatabaseReference = References.getDatabaseReference()
        private var auth: FirebaseAuth = Firebase.auth
        private var rentalAdapter: RentalsAdapter = RentalsAdapter()
        private var myRentalAdapter: MyRentalsAdapter = MyRentalsAdapter()

        fun saveRental(dtoRentals: DtoRentals) {
            val map = HashMap<String, Any>()
            map.put("rentals/${auth.currentUser?.displayName.toString()}/${dtoRentals.title}/owner", dtoRentals.owner)
            map.put("rentals/${auth.currentUser?.displayName.toString()}/${dtoRentals.title}/title", dtoRentals.title)
            map.put("rentals/${auth.currentUser?.displayName.toString()}/${dtoRentals.title}/description", dtoRentals.description)
            map.put("rentals/${auth.currentUser?.displayName.toString()}/${dtoRentals.title}/price", dtoRentals.price)
            map.put("rentals/${auth.currentUser?.displayName.toString()}/${dtoRentals.title}/image", dtoRentals.image)
            map.put("rentals/${auth.currentUser?.displayName.toString()}/${dtoRentals.title}/latitude", dtoRentals.latitude)
            map.put("rentals/${auth.currentUser?.displayName.toString()}/${dtoRentals.title}/longitude", dtoRentals.longitude)

            database.updateChildren(map)
        }

        fun getAllRentals(recyclerView: RecyclerView, noRentalsImage: ImageView) {
            var rentalsRef = References.getDatabaseReference("rentals")

            var list = ArrayList<DtoRentals>()

            rentalsRef.addListenerForSingleValueEvent(object: ValueEventListener {

                override fun onDataChange(snapshot: DataSnapshot) {
                    list.clear()

                    for(item in snapshot.children) {
                        val userDb = auth.currentUser?.displayName
                        if(!item.key.equals(userDb)) {
                            val itemMap: HashMap<String, Objects> = item.value as HashMap<String, Objects>

                            for(items in itemMap) {
                                val map: HashMap<String, Objects> = items.value as HashMap<String, Objects>

                                val owner: String = map.get("owner").toString()
                                val title: String = map.get("title").toString()
                                val description: String = map.get("description").toString()
                                val price: String = map.get("price").toString()
                                val image: String = map.get("image").toString()
                                val latitude: Double = map.get("latitude") as Double
                                val longitude: Double = map.get("longitude") as Double

                                list.add(DtoRentals(owner, title, description, price, image, latitude, longitude))
                            }

                            if(list.isNotEmpty()) noRentalsImage.visibility = View.GONE
                            else noRentalsImage.visibility = View.VISIBLE

                            initRecyclerView(recyclerView, 1)
                        }
                    }
                    rentalAdapter.submitList(list)
                }

                override fun onCancelled(error: DatabaseError) {
                    Log.d(RentalsFragment.TAG, error.message)
                }
            })
        }

        fun getRentalsWithFilter(recyclerView: RecyclerView, noRentalsImage: ImageView, kilometers: Int, myLocation: LatLng) {
            var rentalsRef = References.getDatabaseReference("rentals")

            var list = ArrayList<DtoRentals>()

            rentalsRef.addListenerForSingleValueEvent(object: ValueEventListener {

                override fun onDataChange(snapshot: DataSnapshot) {
                    list.clear()

                    for(item in snapshot.children) {
                        val userDb = auth.currentUser?.displayName
                        if(!item.key.equals(userDb)) {
                            val itemMap: HashMap<String, Objects> = item.value as HashMap<String, Objects>

                            for(items in itemMap) {
                                val map: HashMap<String, Objects> = items.value as HashMap<String, Objects>

                                val owner: String = map.get("owner").toString()
                                val title: String = map.get("title").toString()
                                val description: String = map.get("description").toString()
                                val price: String = map.get("price").toString()
                                val image: String = map.get("image").toString()
                                val latitude: Double = map.get("latitude") as Double
                                val longitude: Double = map.get("longitude") as Double

                                var result = FloatArray(1)
                                Location.distanceBetween(latitude, longitude, myLocation.latitude, myLocation.longitude, result)
                                result[0] = result[0] / 1000

                                if(result[0] < kilometers){
                                    list.add(DtoRentals(owner, title, description, price, image, latitude, longitude))
                                }
                            }

                            if(list.isNotEmpty()) noRentalsImage.visibility = View.GONE
                            else noRentalsImage.visibility = View.VISIBLE

                            initRecyclerView(recyclerView, 1)
                        }
                    }
                    rentalAdapter.submitList(list)
                }

                override fun onCancelled(error: DatabaseError) {
                    Log.d(RentalsFragment.TAG, error.message)
                }
            })
        }


        fun getMyRentals(recyclerView: RecyclerView, noRentalsImage: ImageView) {
            var userRef = References.getDatabaseReference("rentals").child(auth.currentUser?.displayName.toString())

            var list = ArrayList<DtoRentals>()

            userRef.addValueEventListener(object: ValueEventListener {

                override fun onDataChange(snapshot: DataSnapshot) {
                    list.clear()

                    for(item in snapshot.children) {
                        val title: String = item.child("title").value.toString()
                        val description: String = item.child("description").value.toString()
                        val price: String = item.child("price").value.toString()
                        val image: String = item.child("image").value.toString()
                        val latitude: Double = item.child("latitude").value as Double
                        val longitude: Double = item.child("longitude").value as Double

                        list.add(DtoRentals(auth.currentUser?.displayName.toString(), title, description, price, image, latitude, longitude))
                    }

                    if(list.isNotEmpty()) noRentalsImage.visibility = View.GONE
                    else noRentalsImage.visibility = View.VISIBLE

                    initRecyclerView(recyclerView, 2)
                    myRentalAdapter.submitList(list)
                }

                override fun onCancelled(error: DatabaseError) {
                    Log.d(MyRentalsFragment.TAG, error.message)
                }
            })
        }

        private fun initRecyclerView(recyclerView: RecyclerView, type: Int) {
            recyclerView.apply {
                layoutManager = LinearLayoutManager(context)
                val topSpacingDecoration = TopSpacingItemDecoration(0)

                if(type == 1) {
                    rentalAdapter = RentalsAdapter()
                    adapter = rentalAdapter
                } else {
                    myRentalAdapter = MyRentalsAdapter()
                    adapter = myRentalAdapter
                }
                addItemDecoration(topSpacingDecoration)

            }
        }
    }
}