package com.cinema.cinemarental.ui.login

import com.cinema.cinemarental.data.DtoLoggedInUser

/**
 * Class that requests authentication and user information from the remote data source and
 * maintains an in-memory cache of login status and user credentials information.
 */

class LoginRepository(val dataSource: LoginDataSource) {

    // in-memory cache of the loggedInUser object
    var userDto: DtoLoggedInUser? = null
        private set

    val isLoggedIn: Boolean
        get() = userDto != null

    init {
        // If user credentials will be cached in local storage, it is recommended it be encrypted
        // @see https://developer.android.com/training/articles/keystore
        userDto = null
    }

    fun logout() {
        userDto = null
        dataSource.logout()
    }

    fun login(username: String, password: String): Result<DtoLoggedInUser> {
        // handle login
        val result = dataSource.login(username, password)

        if (result is Result.Success) {
            setLoggedInUser(result.data)
        }

        return result
    }

    private fun setLoggedInUser(dtoLoggedInUser: DtoLoggedInUser) {
        this.userDto = dtoLoggedInUser
        // If user credentials will be cached in local storage, it is recommended it be encrypted
        // @see https://developer.android.com/training/articles/keystore
    }
}